require 'push_routes/engine'
require 'push_routes/push_route_url'
require 'push_routes/push_route'
require 'push_routes/helpers'
require 'redis'

#Note: Could consider switching listeners to the 'whisper' pub-sub gem later, instead of attaching directly
# After looking at the gem it would be essentially the same for this project in it's current state
# however if we ever need to have another layer of listener events or have callbacks happen
# asynchronously, it's something we could try

#TODO: setup the add_trigger method to take a block and do the rails magic which will take functions defined with regular names and attach them
# That rails magic might not be possible...
module PushRoutes
  class << self
    attr_accessor :NODE_URL
  end
  @@redis = nil
  @@controllers = Set.new

  def self.newGetRoutes
    hash = {}
    PushRoutes.controllers.each do |c|
      c.push_routes.each do |k,v|
        hash[v.source] = c.controller_path+"#"+k.to_s if v.isNew
      end
    end
    return hash
  end

  def self.REDIS_URL
    @@redis.client.options[:url] if @@redis
  end

  def self.trigger(route, type: '', data: 'placeholder')
    return warn "PushRoutes: Redis not configured properly" if PushRoutes.redis.nil?
    begin
      PushRoutes.redis.publish 'rt-change', {channel: route, type: type, data: data}.to_json
    rescue Redis::CannotConnectError => e
      warn "PushRoutes cannot reach the redis server"
    end
  end

  def self.REDIS_URL= (x)
    begin
      @@redis = Redis.new(:url => x);
    rescue
      warn "Failed to connect to redis server. Push Routes will not work"
    end
  end

  def self.redis
    @@redis
  end

  def self.class_prefixes(string)
    res = []
    pos = 0
    while(string.index("::",pos))
      pos = string.index("::",pos)
      res << string[0..pos-1]
      pos += 2
    end
    res << string[0..-1]
  end

  def self.controllers
    Dir[Rails.root.join('app','controllers','**','*.rb')].map do |file|
      #Note: This may break if the directory structure has /app/controllers multiple times.
      #We want to match everything after the first app/controllers after the project root
      Object.const_get(/^.*?\/app\/controllers\/(.*?).rb$/.match(file)[1].camelize)
    end
    # Dir[Rails.root.join('app','controllers','**','*.rb')].each do |file|
    #   #Note: This may break if the directory structure has /app/controllers multiple times.
    #   #We want to match everything after the first app/controllers after the project root

    #   controller_name = /^.*?\/app\/controllers\/(.*?).rb$/.match(file)[1].camelize
    #   class_prefixes(controller_name)[0..-2].each do |name|
    #     Object.const_get(name)
    #   end
    # end
  end



  def self.active_routes
    PushRoutes.controllers.flat_map {|c| c.push_routes.map {|k,v| v.source}}
  end

  class ActionController::Base
    include PushRoute
  end
end
