module PushRoutes
  class Engine < Rails::Engine
    def self.load_controllers
      if(PushRoutes.redis)
        begin
          PushRoutes.redis.set("PUSH_ROUTES_ACTIVE_ROUTES",PushRoutes.active_routes.to_json)
        rescue
          warn "Redis connection failed"
        end
      else
        warn "Redis server not configured properly"
      end
    end

    initializer "push_routes.view_helpers" do
      ActionView::Base.send :include, Helpers
    end

    initializer 'push_routes.precompile_hook', :group => :all do |app|
      app.config.assets.precompile += ['socket.io.js']
      app.config.assets.precompile += ['push_routes_socket_io.js']
    end

    config.after_initialize do
      if(ENV["REDIS_URL"])
        PushRoutes.REDIS_URL=ENV["REDIS_URL"]
      end

      if(ENV["NODE_URL"])
        PushRoutes.NODE_URL=ENV["NODE_URL"]
      end

      Rails.application.reload_routes!
      PushRoutes::Engine.load_controllers
      ActionDispatch::Reloader.to_prepare do
        #Note: may need to reload routes too? only if the routes changed
        # should never happen in production
        PushRoutes::Engine.load_controllers
      end
    end
  end
end
